@@ -21,9 +21,20 @@
    */
 
 #ifndef HSA_RUNTIME_WRAPPER_INC_HSA_API_TRACE_H
-#define HSA_RUNTIME_WRAPPER_INC_HSA_API_TRACE_H 
+#define HSA_RUNTIME_WRAPPER_INC_HSA_API_TRACE_H
 
+#ifndef ROCM_HEADER_WRAPPER_WERROR
+#define ROCM_HEADER_WRAPPER_WERROR 0
+#endif
+#if ROCM_HEADER_WRAPPER_WERROR  /* ROCM_HEADER_WRAPPER_WERROR 1 */
+#error "hsa_api_trace.h has moved to /opt/rocm/include/hsa and package include paths have changed.\nInclude as \"hsa/hsa_api_trace.h\" when using cmake packages."
+#else  /* ROCM_HEADER_WRAPPER_WERROR 0 */
+#if defined(__GNUC__)
+#warning "hsa_api_trace.h has moved to /opt/rocm/include/hsa and package include paths have changed.\nInclude as \"hsa/hsa_api_trace.h\" when using cmake packages."
+#else
 #pragma message("hsa_api_trace.h has moved to /opt/rocm/include/hsa and package include paths have changed.\nInclude as \"hsa/hsa_api_trace.h\" when using cmake packages.")
+#endif
+#endif  /* ROCM_HEADER_WRAPPER_WERROR */
 #include "../../../include/hsa/hsa_api_trace.h"
 
 
